/***********************************************************************
 * File: buffer_editor.h
 * 	(Extracted from
 *	   "$Id: buffer_editor.h,v 1.3 2009/05/30 22:53:17 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This C header file contains definitions of `buffer_editor' API-s.
 *
 * Copyright (C) 2007 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

#ifndef _BUFFER_EDITOR_H_ /* { */
#define _BUFFER_EDITOR_H_
#ifdef __cplusplus
extern "C" {
#endif

/* buffer_editor needs the bellow system include files */
#include <unistd.h>
#include <limits.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <memory.h>
#include <errno.h>
#include <sys/types.h>
#include <regex.h>
#if (!defined(_SVID_SOURCE) && !defined(_BSD_SOURCE) && (!defined(_XOPEN_SOURCE) || _XOPEN_SOURCE >= 500)) /* { */
	extern char *strdup(const char *s);
#endif /* } */
#if (!defined(_GNU_SOURCE)) /* { */
	extern char *strndup(const char *s, size_t n);
#endif /* } */

#if !defined(REG_BASIC) /* { */
#	define REG_BASIC REG_EXTENDED
#endif /* } !defined(REG_BASIC) */
#if !defined(REG_OKAY) /* { */
#	define REG_OKAY	0
#endif /* } !defined(REG_OKAY) */

/* API of compiling FSM for editing buffer:
 * Arguments:
 *	o `pattern1' point to the regular expression
 *	o `cflags' indicates the `regcomp(3)' compiling flags that is
 *	 `REG_BASIC' or `logical or'-ed value of `REG_EXTENDED' and
 *	 `REG_ICASE'.
 *	o `pattern2' is pointing to substitute the REGEXP string.
 * 	  The substitute description is as like `sed(1)'.
 *	o `pattern2_delimit' specifies the character delimits the sub-
 *	 patterns. if it is nil (\0'), the default character is
 *	 backslash.
 *	o `regcomp_errcode' outputs the return code from `regcomp(3)',
 *	 when this API returns NULL, and the error is caused in the
 *	 regular expression libraries. If the regular expression
 *	 libraries occurred no errors, the output is `REG_OKAY'.
 *	  This value is able to convert the error message string,
 *	 using `regerror(3)'.
 *	o `regcomp_errbuf' outputs the error message of `regcomp(3)',
 *	 if `regcomp_errbuf' isn't NULL.
 *	o `regcomp_errbuf_size' indicates the size of `regcomp_errbuf'.
 *
 * Returns;
 *	o NULL: it means several errors occurred
 *	o non-NULL: it points to a compiled FSM for these API-s.
 */
typedef void *Buffer_edit_FSM_t;
extern Buffer_edit_FSM_t buffer_edit_regcomp(
			  char		*pattern1
			, int			cflags
			, char			*pattern2
			, char			pattern2_delimit
			, int			*regcomp_errcode
			, char			*regcomp_errbuf
			, size_t		regcomp_errbuf_size
			);


/* API of releasing the FSM that was compiled by `buffer_edit_regcomp()'
 * Arguments:
 *	o `regex' is pointing to the result of `buffer_edit_regcomp()'
 *	 (it is a compiled FSM for these API-s).
 *
 * Returns:
 *	o non-zero: it means occurred errors.
 *	o zero: it means normal-end.
 */
extern int buffer_edit_regfree(Buffer_edit_FSM_t	regex);


/* editing command identifiers */
typedef enum buffer_edit_command_e {
	  Buffer_edit_cmd_extract	= 0
	, Buffer_edit_cmd_delete
	, Buffer_edit_cmd_substitute
	, Buffer_edit_cmd_substitute_only
	, Buffer_edit_cmd_general_substitute
	, Buffer_edit_cmd_general_substitute_only
	, Buffer_edit_cmd_N
} Buffer_edit_command_e;

/* API of editing buffer with the compiled FSM:
 * Arguments:
 *	o `src' is pointing to the source buffer.
 *	o `start_line_no' indicates number as start of lines.
 *	 if it is zero, this API assumes the number is one.
 *	o `end_line_no' indicates number as end of lines.
 *	 if it is zero, this API assumes the number is `(size_t)-1'.
 *	o `cmd' indicates the command of editing (extract, delete or
 *	  (non-general or general) substitute several lines in the `src'
 *	 buffer).
 *	o `regex' is pointing to the result of `buffer_edit_regcomp()'
 *	 (it is a compiled FSM for these API-s).
 *
 * Returns:
 *	o NULL: it means several errors occurred. `errno' indicates the
 *	 details of errors.
 *	o non-NULL: it is pointing to an allocated string that contains
 *	 the result of editing.
 */
extern char *buffer_edit_FSM(char		*src
			, size_t		start_line_no
			, size_t		end_line_no
			, Buffer_edit_command_e	cmd
			, Buffer_edit_FSM_t	regex
			);


/* API of editing buffer:
 * Arguments:
 *	o `src' is pointing to the source buffer.
 *	o `start_line_no' indicates number as start of lines.
 *	 if it is zero, this API assumes the number is one.
 *	o `end_line_no' indicates number as end of lines.
 *	 if it is zero, this API assumes the number is `(size_t)-1'.
 *	o `cmd' indicates the command of editing (extract, delete or
 *	  (non-general or general) substitute several lines in the `src'
 *	 buffer).
 *	o `pattern1' is pointing to the REGEXP string.
 *	o `cflags' indicates the `regcomp(3)' compiling flags that is
 *	 `REG_BASIC' or `logical or'-ed value of `REG_EXTENDED' and
 *	 `REG_ICASE'.
 *	o `pattern2' is pointing to substitute the REGEXP string.
 * 	  The substitute description is like as `sed(1)'.
 *	o `pattern2_delimit' specifies the character delimits the sub-
 *	 patterns. if it is nil (\0'), the default character is
 *	 backslash.
 *	o `regcomp_errcode' outputs the return code from `regcomp(3)',
 *	 when this API returns NULL, and the error is caused in the
 *	 regular expression libraries. If the regular expression
 *	 libraries occurred no errors, the output is `REG_OKAY'.
 *	  This value is able to convert the error message string,
 *	 using `regerror(3)'.
 *	o `regcomp_errbuf' outputs the error message of `regcomp(3)',
 *	 if `regcomp_errbuf' isn't NULL.
 *	o `regcomp_errbuf_size' indicates the size of `regcomp_errbuf'.
 *	 
 * Returns:
 *	o NULL: it means several errors occurred. `errno' indicates the
 *	 details of errors.
 *	o non-NULL: it is pointing to the allocated memory that
 *	 contains the result of editing.
 */
extern char *buffer_edit(char			*src
			, size_t		start_line_no
			, size_t		end_line_no
			, Buffer_edit_command_e	cmd
			, char			*pattern1
			, int			cflags
			, char			*pattern2
			, char			pattern2_delimit
			, int			*regcomp_errcode
			, char			*regcomp_errbuf
			, size_t		regcomp_errbuf_size
			);
#define Buffer_edit_pattern_backslash	"\\"


/* API to release the result of `buffer_edit()' and `buffer_edit_FSM()':
 * Arguments:
 *	o `edited_buffer' points to an allocated string that contains
 *	 the result of `buffer_edit()' (or `buffer_edit_FSM()').
 *
 * Returns:
 *	o non-zero: it means several errors occurred.
 *	o zero: it means normal-end.
 */
extern int buffer_edit_free(char	*edited_buffer);

#ifdef __cplusplus
}
#endif
#endif /* } !defined(_BUFFER_EDITOR_H_) */
